@@ -1,10 +1,14 @@
 //! Generic-array based string
 //!
-//! Since rust doesn't have constant generics yet [`typenum`] is used to allow for generic arrays (through `generic-array` crate)
+//! String based on generic array
 //!
-//! Can't outgrow capacity (defined at compile time), always occupies [`capacity`] `+ 1` bytes of memory
+//! Since rust doesn't have constant generics yet [`typenum`] is used to allow for generic arrays ([`U1`] to [`U255`])
 //!
-//! *Doesn't allocate memory on the heap and never panics (all panic branches are stripped at compile time)*
+//! *If you need bigger than [`U255`] open an issue explaining your use-case and we may implement*
+//!
+//! Can't outgrow initial capacity (defined at compile time), always occupies [`capacity`] `+ 1` bytes of memory
+//!
+//! *Doesn't allocate memory on the heap and never panics in release (all panic branches are stripped at compile time - except `Index`/`IndexMut` traits, since they are supposed to)*
 //!
 //! ## Why
 //!
@@ -22,16 +26,22 @@
 //!
 //! [`typenum`]: ../typenum/index.html
 //! [`capacity`]: ./struct.ArrayString.html#method.capacity
+//! [`U1`]: ../typenum/type.U1.html
+//! [`U255`]: ../typenum/type.U255.html
 //!
 //! ## Features
 //!
 //! **default:** `std`
 //!
-//! - `std` enabled by default, enables `std` compatibility (remove it to be `#[no_std]` compatible)
+//! - `std` enabled by default, enables `std` compatibility - `impl Error` trait for errors (remove it to be `#[no_std]` compatible)
 //! - `serde-traits` enables serde traits integration (`Serialize`/`Deserialize`)
 //!
 //!     Opperates like `String`, but truncates it if it's bigger than capacity
 //!
+//!  - `diesel-traits` enables diesel traits integration (`Insertable`/`Queryable`)
+//!
+//!      Opperates like `String`, but truncates it if it's bigger than capacity
+//!
 //! - `logs` enables internal logging
 //!
 //!     You will probably only need this if you are debugging this library
@@ -63,9 +73,9 @@
 //!
 //! ## Licenses
 //!
-//! MIT and Apache-2.0
+//! [`MIT`] and [`Apache-2.0`]
 
-#![doc(html_root_url = "https://docs.rs/arraystring/0.2.0/arraystring")]
+#![doc(html_root_url = "https://docs.rs/arraystring/0.2.1/arraystring")]
 #![cfg_attr(docs_rs_workaround, feature(doc_cfg))]
 #![cfg_attr(not(feature = "std"), no_std)]
 #![warn(
@@ -98,7 +108,7 @@
 )]
 #![doc(test(attr(deny(warnings))))]
 
-pub use generic_array::typenum;
+pub use typenum;
 
 /// Remove logging macros when they are disabled (at compile time)
 #[macro_use]
@@ -112,48 +122,38 @@ mod mock {
     macro_rules! error(($($x:tt)*) => ());
 }
 
-#[cfg(feature = "std")]
-extern crate std as core;
-
-/*
 #[cfg(all(feature = "diesel-traits", test))]
 #[macro_use]
 extern crate diesel;
-*/
 
-mod array;
+mod arraystring;
 pub mod drain;
 pub mod error;
+mod generic;
 mod implementations;
-//#[cfg(any(feature = "serde-traits", feature = "diesel-traits"))]
-#[cfg(feature = "serde-traits")]
+#[cfg(any(feature = "serde-traits", feature = "diesel-traits"))]
 mod integration;
 #[doc(hidden)]
 pub mod utils;
 
 /// Most used traits and data-strucutres
 pub mod prelude {
-    pub use crate::array::ArrayString;
+    pub use crate::arraystring::ArrayString;
     pub use crate::drain::Drain;
     pub use crate::error::{OutOfBounds, Utf16, Utf8};
-    pub use crate::{SmallString, MaxString, CacheString};
-
-    pub use generic_array::typenum;
-    pub(crate) use generic_array::ArrayLength;
+    pub use crate::{generic::Length, CacheString, MaxString, SmallString};
 }
 
-pub use crate::array::ArrayString;
+pub use crate::arraystring::ArrayString;
 pub use crate::error::Error;
 
-use core::fmt::{self, Display, Formatter, Write};
-use core::ops::*;
-use core::{str::FromStr, borrow::Borrow};
-#[cfg(feature = "serde-traits")]
-use serde::{Deserialize, Serialize};
-use generic_array::typenum::{U21, U255, U63, Unsigned};
+use crate::prelude::*;
+use core::fmt::{self, Debug, Display, Formatter, Write};
+use core::{borrow::Borrow, borrow::BorrowMut, ops::*};
+use core::{cmp::Ordering, hash::Hash, hash::Hasher, str::FromStr};
 #[cfg(feature = "logs")]
 use log::trace;
-use crate::prelude::*;
+use typenum::{Unsigned, U21, U255, U63};
 
 /// String with the same `mem::size_of` of a `String` in a 64 bits architecture
 pub type SmallString = ArrayString<U21>;
@@ -165,24 +165,9 @@ pub type MaxString = ArrayString<U255>;
 ///
 /// 63 bytes of string
 #[repr(align(64))]
-#[derive(Copy, Clone, Debug, Default, Hash, Eq, PartialEq, Ord, PartialOrd)]
-#[cfg_attr(feature = "serde-traits", derive(Deserialize, Serialize))]
+#[derive(Copy, Clone, Default)]
 pub struct CacheString(pub ArrayString<U63>);
 
-impl Deref for CacheString {
-    type Target = ArrayString<U63>;
-
-    fn deref(&self) -> &Self::Target {
-        &self.0
-    }
-}
-
-impl DerefMut for CacheString {
-    fn deref_mut(&mut self) -> &mut ArrayString<U63> {
-        &mut self.0
-    }
-}
-
 impl CacheString {
     /// Creates new empty `CacheString`.
     ///
@@ -617,6 +602,58 @@ impl CacheString {
     }
 }
 
+impl Debug for CacheString {
+    #[inline]
+    fn fmt(&self, f: &mut Formatter) -> fmt::Result {
+        f.debug_tuple("CacheString").field(&self.0).finish()
+    }
+}
+
+impl Hash for CacheString {
+    #[inline]
+    fn hash<H: Hasher>(&self, hasher: &mut H) {
+        self.0.hash(hasher);
+    }
+}
+
+impl PartialEq for CacheString {
+    #[inline]
+    fn eq(&self, other: &Self) -> bool {
+        self.0.eq(&other.0)
+    }
+}
+impl Eq for CacheString {}
+
+impl Ord for CacheString {
+    #[inline]
+    fn cmp(&self, other: &Self) -> Ordering {
+        self.0.cmp(&other.0)
+    }
+}
+
+impl PartialOrd for CacheString {
+    #[inline]
+    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
+        Some(self.cmp(other))
+    }
+}
+
+impl Deref for CacheString {
+    type Target = ArrayString<U63>;
+
+    #[inline]
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
+impl DerefMut for CacheString {
+    #[inline]
+    fn deref_mut(&mut self) -> &mut ArrayString<U63> {
+        &mut self.0
+    }
+}
+
 impl Display for CacheString {
     #[inline]
     fn fmt(&self, f: &mut Formatter) -> fmt::Result {
@@ -667,6 +704,12 @@ impl Borrow<str> for CacheString {
         self.0.borrow()
     }
 }
+impl BorrowMut<str> for CacheString {
+    #[inline]
+    fn borrow_mut(&mut self) -> &mut str {
+        self.0.borrow_mut()
+    }
+}
 
 impl<'a> Add<&'a str> for CacheString {
     type Output = Self;
